==> v1.20.tar.gz: wcs/wcs/ctl/hobo_notify.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2014  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import json

from quixote import get_publisher
from wcs.roles import Role
from qommon.ctl import Command
from qommon.publisher import get_cfg
from wcs.qommon.misc import json_encode_helper


class CmdHoboNotify(Command):
    name = 'hobo_notify'

    def execute(self, base_options, sub_options, args):
        self.base_options = base_options
        if sub_options.extra:
            if not self.config.has_section('extra'):
                self.config.add_section('extra')
            for i, extra in enumerate(sub_options.extra):
                self.config.set('extra', 'cmd_line_extra_%d' % i, extra)

        notification = self.load_notification(args)
        if not self.check_valid_notification(notification):
            sys.exit(1)
        import publisher

        publisher.WcsPublisher.configure(self.config)
        pub = publisher.WcsPublisher.create_publisher()
        global_app_dir = pub.app_dir
        for hostname in os.listdir(global_app_dir):
            app_dir = os.path.join(global_app_dir, hostname)
            if not os.path.exists(os.path.join(app_dir, 'config.pck')):
                continue
            pub.app_dir = app_dir
            pub.set_config()
            self.process_notification(notification, pub)

    @classmethod
    def load_notification(cls, args):
        if args[0] == '-':
            # get environment definition from stdin
            return json.load(sys.stdin)
        else:
            return json.load(file(args[0]))

    @classmethod
    def check_valid_notification(cls, notification):
        return isinstance(notification, dict) \
            and notification['@type'] in ['provision', 'deprovision'] \
            and 'objects' in notification \
            and 'audience' in notification \
            and isinstance(notification['audience'], list) \
            and isinstance(notification['objects'], dict) \
            and '@type' in notification['objects'] \
            and 'data' in notification['objects'] \
            and isinstance(notification['objects']['data'], list)

    @classmethod
    def process_notification(cls, notification, publisher=None):
        publisher = publisher or get_publisher()
        action = notification['@type']
        audience = notification['audience']
        full = notification['full'] if 'full' in notification else False
        issuer = notification.get('issuer')

        # Verify tenant is in audience
        entity_id = get_cfg('sp', {}).get('saml2_providerid')
        if not entity_id or entity_id not in audience:
            return

        t = notification['objects']['@type']
        # Now provision/deprovision
        getattr(cls, 'provision_' + t)(publisher, issuer, action,
                                       notification['objects']['data'],
                                       full=full)

    @classmethod
    def check_valid_role(cls, o):
        return 'uuid' in o \
           or 'name' in o \
           or 'description' in o \
           or 'emails' in o \
           or 'emails_to_members' in o \
           or 'slug' in o

    @classmethod
    def provision_role(cls, publisher, issuer, action, data, full=False):
        uuids = set()
        for o in data:
            assert cls.check_valid_role(o)
            uuid = o['uuid'].encode(publisher.site_charset)
            uuids.add(uuid)
            slug = o['slug'].encode(publisher.site_charset)
            details = o.get('description', '').encode(publisher.site_charset)
            name = o['name'].encode(publisher.site_charset)
            emails = [email.encode(publisher.site_charset) for email in o['emails']]
            emails_to_members = o['emails_to_members']
            # Find existing role
            try:
                role = Role.get(uuid)
            except KeyError:
                try:
                    role = Role.get_on_index(uuid, 'slug')
                except KeyError:
                    try:
                        role = Role.get_on_index(slug, 'slug')
                    except KeyError:
                        # New role
                        if action != 'provision':
                            continue
                        role = Role()
                        role.id = uuid
            if action == 'provision':
                # Provision/rename
                role.name = name
                role.slug = uuid
                role.emails = emails
                role.details = details
                role.emails_to_members = emails_to_members
                role.store()
            elif action == 'deprovision':
                # Deprovision
                role.remove_self()
        # All roles have been sent
        if full:
            for role in Role.select():
                if role.slug not in uuids:
                    role.remove_self()

    @classmethod
    def check_valid_user(cls, o):
        return 'uuid' in o \
               and 'email' in o \
               and 'first_name' in o \
               and 'last_name' in o \
               and 'roles' in o

    @classmethod
    def provision_user(cls, publisher, issuer, action, data, full=False):
        from wcs.roles import Role

        User = publisher.user_class

        assert not full # full is not supported for users

        for o in data:
            try:
                o = json_encode_helper(o, publisher.site_charset)
                if action == 'provision':
                    assert cls.check_valid_user(o)
                    uuid = o['uuid']
                    users = User.get_users_with_name_identifier(uuid)
                    assert len(users) < 2  # we are fucked if it's not the case...
                    if users:
                        user = users[0]
                    else:
                        user = User(uuid)
                    user.form_data = user.form_data or {}
                    user.form_data['_email'] = o['email']
                    user.form_data['_first_name'] = o['first_name']
                    user.form_data['_last_name'] = o['last_name']
                    user.name_identifiers = [uuid]
                    role_uuids = [role['uuid'] for role in o['roles']]
                    # reset roles
                    user.roles = []
                    for role_uuid in role_uuids:
                        try:
                            role = Role.get(role_uuid)
                        except KeyError:
                            try:
                                role = Role.get_on_index(role_uuid, 'slug')
                            except KeyError:
                                continue
                        if role.id not in user.roles:
                            user.add_roles([role.id])
                    user.set_attributes_from_formdata(user.form_data)
                    user.store()
                    # verify we did not produce a doublon
                    users = User.get_users_with_name_identifier(uuid)
                    for doublon in users:
                        if int(doublon.id) < int(user.id):  # we are not the first so backoff
                            user.remove_self()
                            break
                elif action == 'deprovision':
                    assert 'uuid' in o
                    uuid = o['uuid']
                    users = User.get_users_with_name_identifier(uuid)
                    for user in users:
                        user.remove_self()
            except Exception, e:
                publisher.notify_of_exception(sys.exc_info(), context='[PROVISIONNING]')

CmdHoboNotify.register()

==> v1.22.tar.gz: wcs/wcs/ctl/hobo_notify.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2014  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import json

from quixote import get_publisher
from wcs.roles import Role
from qommon.ctl import Command
from qommon.publisher import get_cfg
from wcs.qommon.misc import json_encode_helper


class CmdHoboNotify(Command):
    name = 'hobo_notify'

    def execute(self, base_options, sub_options, args):
        self.base_options = base_options
        if sub_options.extra:
            if not self.config.has_section('extra'):
                self.config.add_section('extra')
            for i, extra in enumerate(sub_options.extra):
                self.config.set('extra', 'cmd_line_extra_%d' % i, extra)

        notification = self.load_notification(args)
        if not self.check_valid_notification(notification):
            sys.exit(1)
        import publisher

        publisher.WcsPublisher.configure(self.config)
        pub = publisher.WcsPublisher.create_publisher()
        global_app_dir = pub.app_dir
        for hostname in os.listdir(global_app_dir):
            app_dir = os.path.join(global_app_dir, hostname)
            if not os.path.exists(os.path.join(app_dir, 'config.pck')):
                continue
            pub.app_dir = app_dir
            pub.set_config()
            self.process_notification(notification, pub)

    @classmethod
    def load_notification(cls, args):
        if args[0] == '-':
            # get environment definition from stdin
            return json.load(sys.stdin)
        else:
            return json.load(file(args[0]))

    @classmethod
    def check_valid_notification(cls, notification):
        return isinstance(notification, dict) \
            and notification['@type'] in ['provision', 'deprovision'] \
            and 'objects' in notification \
            and 'audience' in notification \
            and isinstance(notification['audience'], list) \
            and isinstance(notification['objects'], dict) \
            and '@type' in notification['objects'] \
            and 'data' in notification['objects'] \
            and isinstance(notification['objects']['data'], list)

    @classmethod
    def process_notification(cls, notification, publisher=None):
        publisher = publisher or get_publisher()
        action = notification['@type']
        audience = notification['audience']
        full = notification['full'] if 'full' in notification else False
        issuer = notification.get('issuer')

        # Verify tenant is in audience
        entity_id = get_cfg('sp', {}).get('saml2_providerid')
        if not entity_id or entity_id not in audience:
            return

        t = notification['objects']['@type']
        # Now provision/deprovision
        getattr(cls, 'provision_' + t)(publisher, issuer, action,
                                       notification['objects']['data'],
                                       full=full)

    @classmethod
    def check_valid_role(cls, o):
        return 'uuid' in o \
           or 'name' in o \
           or 'description' in o \
           or 'emails' in o \
           or 'emails_to_members' in o \
           or 'slug' in o

    @classmethod
    def provision_role(cls, publisher, issuer, action, data, full=False):
        uuids = set()
        for o in data:
            assert cls.check_valid_role(o)
            uuid = o['uuid'].encode(publisher.site_charset)
            uuids.add(uuid)
            slug = o['slug'].encode(publisher.site_charset)
            details = o.get('description', '').encode(publisher.site_charset)
            name = o['name'].encode(publisher.site_charset)
            emails = [email.encode(publisher.site_charset) for email in o['emails']]
            emails_to_members = o['emails_to_members']
            # Find existing role
            try:
                role = Role.get(uuid)
            except KeyError:
                try:
                    role = Role.get_on_index(uuid, 'slug')
                except KeyError:
                    try:
                        role = Role.get_on_index(slug, 'slug')
                    except KeyError:
                        # New role
                        if action != 'provision':
                            continue
                        role = Role()
                        role.id = uuid
            if action == 'provision':
                # Provision/rename
                role.name = name
                role.slug = uuid
                role.emails = emails
                role.details = details
                role.emails_to_members = emails_to_members
                role.store()
            elif action == 'deprovision':
                # Deprovision
                role.remove_self()
        # All roles have been sent
        if full:
            for role in Role.select():
                if role.slug not in uuids:
                    role.remove_self()

    @classmethod
    def check_valid_user(cls, o):
        return 'uuid' in o \
               and 'email' in o \
               and 'first_name' in o \
               and 'last_name' in o \
               and 'roles' in o

    @classmethod
    def provision_user(cls, publisher, issuer, action, data, full=False):
        from wcs.roles import Role

        User = publisher.user_class

        assert not full # full is not supported for users

        for o in data:
            try:
                o = json_encode_helper(o, publisher.site_charset)
                if action == 'provision':
                    assert cls.check_valid_user(o)
                    uuid = o['uuid']
                    users = User.get_users_with_name_identifier(uuid)
                    assert len(users) < 2  # we are fucked if it's not the case...
                    if users:
                        user = users[0]
                    else:
                        user = User(uuid)
                    user.form_data = user.form_data or {}
                    for key in o:
                        if key in ['uuid', 'roles']:
                            continue
                        user.form_data['_%s' % key] = o[key]
                    user.name_identifiers = [uuid]
                    role_uuids = [role['uuid'] for role in o['roles']]
                    # reset roles
                    user.roles = []
                    for role_uuid in role_uuids:
                        try:
                            role = Role.get(role_uuid)
                        except KeyError:
                            try:
                                role = Role.get_on_index(role_uuid, 'slug')
                            except KeyError:
                                continue
                        if role.id not in user.roles:
                            user.add_roles([role.id])
                    user.set_attributes_from_formdata(user.form_data)
                    user.store()
                    # verify we did not produce a doublon
                    users = User.get_users_with_name_identifier(uuid)
                    for doublon in users:
                        if int(doublon.id) < int(user.id):  # we are not the first so backoff
                            user.remove_self()
                            break
                elif action == 'deprovision':
                    assert 'uuid' in o
                    uuid = o['uuid']
                    users = User.get_users_with_name_identifier(uuid)
                    for user in users:
                        user.remove_self()
            except Exception, e:
                publisher.notify_of_exception(sys.exc_info(), context='[PROVISIONNING]')

CmdHoboNotify.register()
